﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using Infragistics.Olap;
using Infragistics.Olap.FlatData;
using Infragistics.Controls.Grids;
using System.Collections;
using System.Reflection;

namespace PivotGridUserControl
{
    /// <summary>
    /// Interaction logic for PivotGridHostControl.xaml
    /// </summary>
    public partial class PivotGridHostControl : UserControl
    {
        public XamPivotDataSelector DataSelector
        {
            get
            {
                return xamPivotDataSelector;
            }
        }

        public PivotGridHostControl()
        {
            InitializeComponent();
        }

        public void SetFlatDatasourceTest(string displayName, DataTable dataTable)
        {
            // ---------- create test table ----------
            dataTable = new DataTable();

            dataTable.Columns.Add("RecordNo", typeof(int));
            dataTable.Columns.Add("TestNo", typeof(int));
            dataTable.Columns.Add("MethodNo", typeof(string));
            dataTable.Columns.Add("IK [ml]", typeof(double));
            dataTable.Columns.Add("EK [ml/g]", typeof(string));

            AddNewRow(dataTable, 134, 34, "M41", 0.45, "<4.52");
            AddNewRow(dataTable, 231, 12, "M77", 0.45, "5.51E-05");
            AddNewRow(dataTable, 221, 24, "M77", 0.45, "<4.98E-05");
            AddNewRow(dataTable, 432, 34, "M41", 0.22, "1.07E-06");
            AddNewRow(dataTable, 9867, 34, "M41", 0.45, ">3.07E-07");
            AddNewRow(dataTable, 28, 12, "M77", 0.45, "1.82E-02");
            AddNewRow(dataTable, 78, 63, "M77", 0.45, "0.08E-07");

            // ---------- FlatDataSource for xamPivotGrid ----------
            DynamicTypeBuilder typeBuilder = new DynamicTypeBuilder
            {
                DynamicAssemblyName = "BI.Assembly",
                DynamicTypeName = displayName
            };

            IList<DynamicTypePropertyInfo> properties = new List<DynamicTypePropertyInfo>();

            foreach (DataColumn column in dataTable.Columns)
            {
                DynamicTypePropertyInfo propertyInfo = new DynamicTypePropertyInfo
                {
                    PropertyName = column.ColumnName,
                    PropertyType = column.DataType
                };

                properties.Add(propertyInfo);
            }

            Type dynamicType = typeBuilder.GenerateType(properties);

            Type listType = typeof(List<>);

            Type genericListType = listType.MakeGenericType(dynamicType);

            IList list = (IList)Activator.CreateInstance(genericListType);

            int i = 0;
            foreach (DataRow dataRow in dataTable.Rows)
            {
                object myDynamicInstance = Activator.CreateInstance(dynamicType);

                foreach (DataColumn column in dataTable.Columns)
                {
                    PropertyInfo propertyVal = dynamicType.GetProperty(column.ColumnName);

                    if (dataRow[column] != DBNull.Value)
                        propertyVal.SetValue(myDynamicInstance, dataRow[column], null);
                }

                list.Add(myDynamicInstance);
            }

            FlatDataSource flatDataSource = new FlatDataSource()
            {
                ItemsSource = list,
                Cube = FlatDataSource.GenerateInitialCube(displayName),
                Columns = FlatDataSource.GenerateInitialItems("[MethodNo].[MethodNo], [TestNo].[TestNo]"),
                Rows = FlatDataSource.GenerateInitialItems("[RecordNo].[RecordNo]"),
                Measures = FlatDataSource.GenerateInitialItems("IK [ml], [EK [ml/g]]")
            };

            xamPivotGrid.DataSource = flatDataSource;
            xamPivotDataSelector.DataSource = flatDataSource;            
        }

        private void AddNewRow(DataTable dataTable, int recordNo, int testNo, string methodNo, double ik, string ek)
        {
            DataRow dataRow = dataTable.NewRow();
            dataRow[0] = recordNo; dataRow[1] = testNo; dataRow[2] = methodNo; dataRow[3] = ik; dataRow[4] = ek;
            dataTable.Rows.Add(dataRow);
        }


    }
}
